function distance = AttributeDistance(Expert1HFS, Expert2HFS, weights)
%ATTRIBUTEDISTANCE distance between attributes
%Expert1HFS Evaluation Matrix of Expert 1
%Expert2HFS Evaluation Matrix of Expert 2
%weights Weight of each attribute
%Average of the sum of the differences of all membership degrees
    N = length(Expert1HFS); %Gets the number of attributes
    d = 0;
    for i = 1 : N
        %Converting the attribute evaluation cell structure into an array
        HFS1 = cell2mat(Expert1HFS(i));
        HFS2 = cell2mat(Expert2HFS(i));
        HN1 = length(HFS1); %Obtain the hesitancy degree of expert 1 attribute I
        HN2 = length(HFS2); %Obtain the hesitancy degree of expert 2 attribute I
        fillNum = HN1 - HN2;
        
        %Completing the hesitating fuzzy element set with smaller hesitating degree
        if fillNum > 0  %Expert 2 has little hesitation and needs to be filled
                HN = HN1;
                NFN = fillNum; %Number of membership degrees to be filled
                for k = 1:NFN
                    Num = mean(HFS2(:));  %Value to be populated
                    HFS2 = [HFS2,Num];
                end
        elseif fillNum < 0  %Expert 1 has little hesitation and needs to be filled.
                HN = HN2;
                NFN = -fillNum; %Number of membership degrees to be filled
                for k = 1:NFN
                    Num = mean(HFS1(:));  %Value to be populated
                    HFS1 = [HFS1,Num];
                end
        else
            %No action required
            HN = HN2;
        end
       
        %Calculate the distance
        sumD = 0;
        for j = 1 : HN
            sumD = sumD + abs(HFS1(j) - HFS2(j)); 
        end
        d = d + weights(i) * (sumD / HN);   
    end
    distance = d;
end

