function [cluster,Dham] = Clusting( HFSs, nearN,  Center)
%CLUSTING Clustering
%   Detailed instructions go here
    time1=clock; %Main Program

    ExpertN = length(HFSs); %Get the total number of experts
    N = length(HFSs{1});  %Gets the number of attributes
    weights = zeros(1,N); %Initialize attribute weight matrix
    waverages = zeros(1,N); %Average value of attribute membership
    CVs = zeros(1,N); %Coefficient of variation

    %And calculate that average value of the membership degree
     for j = 1:N
         n = 0; %Total degree of affiliation
         sumL = 0; %Total membership value
         for i = 1:ExpertN
             HFS = cell2mat(HFSs{i}(j));
             for k = 1:length(HFS)
                 sumL = sumL + HFS(k);
                 n = n + 1;
             end
         end
         a = sumL / n;
         waverages(j) = a;
     end

     %Calculate the standard deviation of membership and coefficient of variation
      for j = 1:N
         nn = 0; %The number of standard deviations
         sumS = 0; %Sum of standard deviations
         for i = 1:ExpertN
             HFS = cell2mat(HFSs{i}(j));
             for k = 1:length(HFS)
                 nn = nn + 1; 
                 sd = power(abs(HFS(k) - waverages(j)),2);
                 sumS = sumS + sd;
             end
         end
         a = sqrt(sumS / nn); %standard deviation
       %  sds(j) = a;
         CV = a / waverages(j); %Coefficient of variation
         CVs(j) = CV;
      end

      %Calculate Attribute Weight
      sumCv = sum(CVs);
      for i = 1:N
          weights(i) = CVs(i) / sumCv;
      end

      %Calculate hesitation-weighted Hamming distances
      Dham = zeros(ExpertN); %Distance matrix between experts
      for i = 1 : ExpertN
          for j = 1 : ExpertN
              if i >= j
                  Dham(i,j) = Dham(j,i);
              else
                  Dham(i,j) = AttributeDistance(HFSs{i},HFSs{j},weights);  
              end
          end
      end

     %Calculate the density
     K = nearN;
     P = zeros(1,ExpertN);
     for i = 1 : ExpertN
         sumP = 0;
         [D,In] = sort(Dham(i,:));

         for j = 1 : K + 1
             if D(j) == 0
                 continue
             end
             sumP = sumP + exp(-D(j));
         end
         P(i) = sumP;
     end

     %Calculate the relative distance
     [Y,I] = sort(P);
     QD = zeros(1,ExpertN);
     for i = 1 : ExpertN
         minDham = inf;
         index = find(I == i);
         if(index == ExpertN)
             minDham = max(Dham(i,:));
         else
             for j = index + 1 : ExpertN
                 minDham = min(minDham, Dham(i,I(j)));
             end
         end
         QD(i) = minDham;
     end

     T = zeros(1,ExpertN);
     for i = 1 : ExpertN
         T(i) = P(i) * QD(i);
     end

    [TT , Ti] = sort(T,'descend');
    disp(TT);
    %Center = input('please choose clusting center nums:');
    cluster = zeros(Center,ExpertN); %Clustering results, multi-point application space, easy to operate
    clusterCenter = zeros(1,Center); %Cluster center density
    for i = 1 : Center
        cluster(i,2) = Ti(i);
        cluster(i,1) = 2;
        clusterCenter(i) = P(Ti(i));
    end

    for i =  Center + 1 : ExpertN
        mayNode = find(P(Ti(i)) <= clusterCenter(:)); %Find the center of the cluster that is denser than it

        %The smallest distance is selected among the possible cluster centers.
        minD = inf; %Minimum distance
        minIndex = 1; %The subscript corresponding to the minimum distance
        for j = 1 : length(mayNode)
            if Dham(Ti(i),Ti(mayNode(j))) < minD
                minD = Dham(Ti(i),Ti(mayNode(j)));
                minIndex = mayNode(j);
            end
        end
        cluster(minIndex,1) = cluster(minIndex,1) + 1;
        cluster(minIndex,cluster(minIndex,1)) = Ti(i);
    end
    time2=clock;%Time difference
    run_time=etime(time2,time1);
    disp(run_time);
end

