%% The core procedure of FCM is to iteratively solve the cluster center. 
%k Number of clusters
%ion Maximum number of iterations
function [centroids, cluster] = FCM( dataSet, k, ion)
    N = length(dataSet);
    sData = [];
    for i = 1:N 
        sData = [sData, dateInit(dataSet{i})];
    end
    clus = kmeans(sData', k);
    cluster = getCluster(clus, k)
    centroids = getCenters(dataSet, cluster, k);
  end

function date = dateInit(dataSet)
    N = length(dataSet);
    sum = 0;
    for i = 1 : N
        HFS = dataSet{i};
        sum = sum + mean(HFS);
    end 
    date = sum / N;
end

function c = getCluster(cluster, k) 
    c = initCluster(k, length(cluster));
    for i = 1 : length(cluster)
       for j = 1 : k
           if cluster(i) == j
                locate = c(j,1) + 1;
                c(j,locate) = i;
                c(j,1) = locate;
           end
       end
    end
end

%Initialize the clustering matrix
function [cluster] = initCluster(k, n)
    cluster = zeros(k, n+1);
    for i = 1 : k
        cluster(i,1) = 1;
    end
end

function [centroids] = getCenters(dataSet, cluster, k)
    centroids = cell(k,1); 
    for i = 1 : k
        c = randi(cluster(i, 1) - 1);
        centroids(i,1) = dataSet(c+1,1);
    end
end

