function HFS = FillLostValue(HFSs , numExpert, lostAttribute, TrustValue)
global type
global param
%FillLostValue Missing value estimation
%HFSs Expert Evaluation Matrix
%numExpert Expert number to be filled
%lostAttribute Missing attribute collection
%TrustValue Comprehensive trust value
%HFS   Return value Expert Hesitation Fuzzy Evaluation Matrix Filled with Missing Values
    neighbors = CollaborativeFiltering(TrustValue, numExpert, type, param);
    
    HFS = HFSs{numExpert};
    for i = 1 : length(lostAttribute)
        HFS{lostAttribute(i)} = FillValue2(HFSs, neighbors, lostAttribute(i));
    end
end

%Using collaborative filtering to fill
function value = FillValue(HFSs, neighbors, numAttribute)
    value = [];
    N = length(neighbors); %Number of neighbor nodes
    newHFSs = cell(N, 1); %Initializing neighbor evaluation matrix
    %Acquire a neighbor evaluation matrix
    for z = 1 : N
        newHFSs{z} = HFSs{neighbors(z)};
    end
    %Indecision alignment
    nHFSs = alignElement(newHFSs, numAttribute);
    if isempty(nHFSs)
        return
    end
    hesitant = length(cell2mat(nHFSs{1}(numAttribute)));
    for i = 1 : hesitant
        v = 1;
        expertNum = length(nHFSs);
        for j = 1 : expertNum
            v = v * ((1 - nHFSs{j}{numAttribute}(i)) ^ (1 / expertNum));
        end
        value = [value,1 - v];
    end
end


%Align hesitancy to facilitate calculation
function nHFSs = alignElement(newHFSs, numAttribute)
    maxHesitant = findMaxHesitant(newHFSs, numAttribute);
    for j = 1 : length(newHFSs)
        attrbutSet = cell2mat(newHFSs{j}(numAttribute));
        if isempty(attrbutSet)
            continue
        end
        if length(attrbutSet) == maxHesitant
            continue
        else
            fillValue = sum(attrbutSet) / length(attrbutSet);
            for k = 1 : maxHesitant - length(attrbutSet)
                attrbutSet = [attrbutSet, fillValue];
            end
        end
        newHFSs{j}{numAttribute} = attrbutSet;
    end 
    %Remove meaningless neighbors
    nHFSs = {};
    for i = 1 : length(newHFSs)
        if ~isempty(cell2mat(newHFSs{i}(numAttribute)))
            nHFSs{end + 1,1} = newHFSs{i};
        end
    end
end

%Get the most indecision
function maxHesitant = findMaxHesitant(newHFSs, numEleme)
    hsize =  length(newHFSs);
    maxHesitant = length(newHFSs{1}(numEleme));
    for i = 1 : hsize
        Hesitant = length(cell2mat(newHFSs{i}(numEleme)));
        maxHesitant = max(maxHesitant,Hesitant);
    end
end
