%k Number of clusters
%ion Maximum number of iterations
function [centroids, cluster, weight] = Zhang( dataSet, k, ion)
    startTime = clock; %Program start time
    [m,n] = size(dataSet);
    lastCluster = [];
    %Initialize the cluster center  
    [centroids, cluster] = randCent(dataSet, k);
    weight = getWeight(dataSet); 
    for j = 1: ion
        for i = 1 : m
            index = chooseNearCenter(centroids, dataSet{i}, weight);
            if cluster(index,2) == i
                continue
            end
            locate = cluster(index,1) + 1;
            cluster(index,1) = locate;
            cluster(index,locate) = i;
        end
        cluster
        if  isChange(cluster, lastCluster) == 0
             return
        end
        lastCluster = cluster;
        [centroids, cluster] = changeCenter(cluster , dataSet);
    end
    endTime = clock;
    time = endTime - startTime;
end

%Sort each row in the matrix except for the first element
function B = partSort( A )
    [len, ~] = size(A);
     for i = 1 : len
        a = A(i, 2:A(i,1));
        b = sort(a);
        A(i, 2:A(i,1)) = b;
     end
     B = A;
end

%Judge whether it is necessary to continue iteration and change the cluster center
function change = isChange(cluster, lastCluster)
     if  isequal(partSort(cluster),partSort(lastCluster))
         change = 0;
         return
     end
     change = 1;
end

%% Take the random center  
function [ centroids, cluster ] = randCent( dataSet, k )  
	[m,~] = size(dataSet);%Obtain the number of decision makers
	centroids = cell(k,1); 
    center = [];
    for i = 1 : k
        c = randi(m,1);
        while(isExist(center, c) == 1)
             c = randi(m,1);
        end
        center = [center, c];
        centroids(i,1) = dataSet(c,1);
    end
    cluster = initCluster(k, m, center);
end 

%Determine whether the randomly generated center already exists
function [ Exist ] = isExist(center, c)
    for i = 1 : length(center)
        if center(i) == c
            Exist = 1;
            return
        end
    end
    Exist = 0;
end

%Get Attribute Weight
function [weights] = getWeight(HFSs)
    ExpertN = length(HFSs); %Get the total number of experts
    N = length(HFSs{1});  %Gets the number of attributes
    weights = zeros(1,N); %Initialize attribute weight matrix
    waverages = zeros(1,N); %Average value of attribute membership
    CVs = zeros(1,N); %Coefficient of variation

    %And calculate that average value of the membership degree
     for j = 1:N
         n = 0; %Total degree of affiliation
         sumL = 0; %Total membership value
         for i = 1:ExpertN
             HFS = cell2mat(HFSs{i}(j));
             for k = 1:length(HFS)
                 sumL = sumL + HFS(k);
                 n = n + 1;
             end
         end
         a = sumL / n;
         waverages(j) = a;
     end

     %Calculate the standard deviation of membership and coefficient of variation
      for j = 1:N
         nn = 0; %The number of standard deviations
         sumS = 0; %Sum of standard deviations
         for i = 1:ExpertN
             HFS = cell2mat(HFSs{i}(j));
             for k = 1:length(HFS)
                 nn = nn + 1; 
                 sd = power(abs(HFS(k) - waverages(j)),2);
                 sumS = sumS + sd;
             end
         end
         a = sqrt(sumS / nn); %standard deviation
       %  sds(j) = a;
         CV = a / waverages(j); %Coefficient of variation
         CVs(j) = CV;
      end

      %Calculate Attribute Weight
      sumCv = sum(CVs);
      for i = 1:N
          weights(i) = CVs(i) / sumCv;
      end
end
  
%Initialize the clustering matrix
function [cluster] = initCluster( k, n, center)
    cluster = zeros(k, n+1);
    for i = 1 : k
        cluster(i,1) = 2;
        cluster(i,2) = center(i);
    end
end

%Select Nearest Center
function [index] = chooseNearCenter(centroids, HFS, weight)
    loop = length(centroids);
    minLen = inf;
    for i = 1 : loop
         distance = AttributeDistance(centroids{i}, HFS, weight);
         if minLen > distance
             minLen = distance;
             index = i;
         end
    end
end

%Align hesitancy to facilitate calculation
function nHFSs = alignElement(newHFSs, numAttribute)
    maxHesitant = findMaxHesitant(newHFSs, numAttribute);
    for j = 1 : length(newHFSs)
        attrbutSet = cell2mat(newHFSs{j}(numAttribute));
        if isempty(attrbutSet)
            continue
        end
        if length(attrbutSet) == maxHesitant
            continue
        else
            fillValue = sum(attrbutSet) / length(attrbutSet);
            for k = 1 : maxHesitant - length(attrbutSet)
                attrbutSet = [attrbutSet, fillValue];
            end
        end
        newHFSs{j}{numAttribute} = attrbutSet;
    end 
    %Remove meaningless neighbors
    nHFSs = {};
    for i = 1 : length(newHFSs)
        if ~isempty(cell2mat(newHFSs{i}(numAttribute)))
            nHFSs{end + 1,1} = newHFSs{i};
        end
    end
end

%Get the most indecision
function maxHesitant = findMaxHesitant(newHFSs, numEleme)
    hsize =  length(newHFSs);
    maxHesitant = length(newHFSs{1}(numEleme));
    for i = 1 : hsize
        Hesitant = length(cell2mat(newHFSs{i}(numEleme)));
        maxHesitant = max(maxHesitant,Hesitant);
    end
end

function  [centroids, cluster] = changeCenter( Cluster, HFSs ) 
     [num, col] = size(Cluster);  %Number of clusters
     centroids = {};
     numAttribute = length(HFSs{1}); %Number of attributes
     for i = 1 : num
        N = Cluster(i,1);
        newHFSs = cell(N-1, 1); %Initializing cluster evaluation matrix
         %Obtaining a cluster evaluation matrix
        for z = 2 : N
            newHFSs{z-1} = HFSs{Cluster(i,z)};
        end
        %Indecision alignment
        HFS = cell(numAttribute,1);
        for j = 1 : numAttribute
            nHFSs = alignElement(newHFSs, j);
            hesitant = length(cell2mat(nHFSs{1}(j)));
            value = [];
            for k = 1 : hesitant
                v = 1;
                expertNum = length(nHFSs);
                for m = 1 : expertNum
                     v = v * ((1 - nHFSs{m}{j}(k)) ^ (1 / expertNum));
                end
                value = [value,1 - v];
            end
            HFS{j} = value;
        end
       centroids{end+1, 1} = HFS;
     end
     cluster = zeros(num,col);
     cluster(:,1) = 1;
end



