function possiablePaths = findPath(Graph, partialPath, destination, partialWeight,isValid)
% findPathSearch depth-first for all possible paths from partialPath to destination that do not contain loops
% Graph: In the road network graph, non-infinite or 0 means that two nodes are directly connected, and the matrix value is the road network weight
% partialPath: The starting path. If partialPath is a number, it means that this is the starting point.
% destination: Destination node
% partialWeight: Weight of the partialpath. When the partialpath is a number, the partialweight is 0
%isValid Can be an array of relay nodes

deepth = 6;% deepth The depth of the recursion, which controls the number of propagation layers 
targetScore = 0.9; %The threshold at which trust continues to propagate

pathLength = length(partialPath); 
lastNode = partialPath(pathLength); %Get the last node
nextNodes = find(0<Graph(lastNode,:) & Graph(lastNode,:)<inf); %Get the next node of the last node according to the Graph
GLength = length(Graph);
possiablePaths = [];
if lastNode == destination
    % If the lastNode is equal to the target node, it means that the partialPath is the path from it to the target node, and the result is only this one, which is returned directly.
    possiablePaths = partialPath;
    possiablePaths(GLength + 1) = partialWeight;
return;
elseif length( find( partialPath == destination ) ) ~= 0
    return;
end
%The number in nextNodes must be greater than 0, so in order for nextNodes (I) to be removed, assign it to 0 first
for i=1:length(nextNodes)
    if destination == nextNodes(i)
      %Output path
      tmpPath = cat(2, partialPath, destination);      %Concatenate into a complete path
   %   tmpPath(GLength + 1) = partialWeight + Graph(lastNode, destination); %Extend the length of the array to GLength + 1, and the last element is used to store the total resistance of the path.
      tmpPath(GLength + 1) = (partialWeight * getCreditScore(Graph,nextNodes(i))) / 4;
      possiablePaths( length(possiablePaths) + 1 , : ) = tmpPath;
      nextNodes(i) = 0;
    elseif length( find( partialPath == nextNodes(i) ) ) ~= 0
      nextNodes(i) = 0;
    end
end
nextNodes = nextNodes(nextNodes ~= 0); %Drop the value of 0 in nextNodes because the next node may have already been traversed or it is the target node.
for i=1:length(nextNodes)
    tmpPath = cat(2, partialPath, nextNodes(i));
    %Controlling the Length of the Trust Propagation Path
    if length(tmpPath) >= deepth
        continue
    end
    % Control the selection of relay nodes and skip the experts with more negative trust relationships
    if isValid(nextNodes(i)) ~= true
        continue
    end
    if partialWeight < targetScore
        continue
    end
    tmpPsbPaths = findPath(Graph, tmpPath, destination, partialWeight * getCreditScore(Graph,nextNodes(i)),isValid);
    possiablePaths = cat(1, possiablePaths, tmpPsbPaths);
end
%Test case
% a=[0,62,66,inf,inf,inf,inf;
%       62,0,inf,25,11,inf,inf;
%       66,inf,0,9,inf,inf,49;
%       inf,25,9,0,11,14,inf;
%       inf,11,inf,11,0,13,inf;
%       inf,inf,inf,14,13,0,35.8;
%       inf,inf,49,inf,inf,35.8,0;];
%Call the M file that searches for all paths between any two points in the graph
%findPath(a, 1, 7, 0)
%isValidNode = [true,true,true,true,false,true,true;];
